import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import gobject
import pango
import gio

try:
    import pynotify
except:
    pass

import urllib

from commandline import *
from kilikalingdbus import *
from database import *
from mediaitem import *
from media import *
from mediaimporter import *
from playlist import *
from medialisttreeview import *
from preferences import *

VERSION = "0.2"

class KilikaliNG:
    def __init__(self, cmdline):
        self.cmdline = cmdline

        self.basicitems = (
            ('File', None, '_File'),
            ('Add', gtk.STOCK_ADD, 'A_dd files', None, 'Add files...', self.addFilesCb),
            ('AddDir', gtk.STOCK_ADD, 'Add d_irectory', None, 'Add files from directory...', self.addDirectoryCb),
            ('AddURI', gtk.STOCK_ADD, 'Add _URI', None, 'Add URI...', self.addURICb),
            ('Quit', gtk.STOCK_QUIT, '_Quit', None, 'Quit the application...', self.deleteEventIdle),
            ('Edit', None, '_Edit'),
            ('Remove', gtk.STOCK_REMOVE, '_Remove', None, 'Remove files from playlist...', self.removeCb),
            ('Prefs', gtk.STOCK_PREFERENCES, '_Preferences', None, 'Preferences...', self.prefsCb),
            ('Clear', gtk.STOCK_CLEAR, '_Clear', None, 'Clear playlist...', self.clearCb),
            ('Help', None, '_Help'),
            ('About', gtk.STOCK_ABOUT, '_About', None, 'About the application...', self.aboutCb),

            ('Playback', None, 'P_layback'),
            ('Play', gtk.STOCK_MEDIA_PLAY, '_Play', None, 'Play...', self.playCb),
            ('Pause', gtk.STOCK_MEDIA_PAUSE, 'P_ause', None, 'Pause...', self.pauseCb),
            ('Stop', gtk.STOCK_MEDIA_STOP, '_Stop', None, 'Stop...', self.stopCb),
            ('Prev', gtk.STOCK_MEDIA_PREVIOUS, 'P_rev', None, 'Previous...', self.prevCb),
            ('Next', gtk.STOCK_MEDIA_NEXT, '_Next', None, 'Next...', self.nextCb),
            ('Jump', None, '_Jump to current', None, 'Jump to current...', self.jumpCb),
            )

        self.toggleactions = (
            ('Repeat', None, 'R_epeat', None, 'Repeat mode...', self.repeatCb, False),
            ('Suffle', None, '_Suffle', None, 'Suffle mode...', self.suffleCb, False),
            )

        # list of stuff inside
        self.pause_mode = False
        self.album_label = None
        self.artist_label = None
        self.title_label = None
        self.window = None
        self.medialist= None
        self.last_dir = None
        self.media = None
        self.previous_mode = ""
        self.seeking = False
        self.seek_to = 0
        self.dbus = 0
        self.notification = None

        self.dbus = KilikaliNGDBus ()
        self.dbus.obj.connect ("quit", self.dbusQuitCb)
        self.dbus.obj.connect ("add-uri-list", self.dbusAddUriListCb)
        self.dbus.obj.connect ("control-play", self.dbusControlPlayCb)
        self.dbus.obj.connect ("control-stop", self.dbusControlStopCb)
        self.dbus.obj.connect ("control-pause", self.dbusControlPauseCb)
        self.dbus.obj.connect ("control-previous", self.dbusControlPreviousCb)
        self.dbus.obj.connect ("control-next", self.dbusControlNextCb)

        self.prefs = Preferences ()
        self.last_dir = self.prefs.getDirectoryPath ()

        configdir = self.prefs.createConfigDir ()
        self.db = Database (configdir, "playlist.db")

        self.media = Media ()
        self.media.connect ("time", self.setCurrentTimeCb, None)
        self.media.connect ("eos", self.eosCb, None)
        self.media.connect ("error", self.errorCb, None)
        self.media.connect ("tag-update", self.tagUpdateCb, None)

        volume = self.prefs.getVolume ()
        self.media.setVolume (volume)

        glib.set_application_name ("KilikaliNG")

        # create window
        x, y, width, height = self.prefs.getWindowAllocation ()
        self.window = gtk.Window (gtk.WINDOW_TOPLEVEL)
        self.window.set_size_request (400,300)
        self.window.move (x, y)
        self.window.resize (width, height)
        self.window.set_title ("KilikaliNG")

        try:
            self.window.get_settings ().set_property ("gtk-button-images", True)
        except:
            pass
        try:
            theme = gtk.icon_theme_get_default ()
            icon = theme.load_icon ("kilikaling", 32, gtk.ICON_LOOKUP_NO_SVG )
            self.window.set_icon (icon)
        except:
            pass

        # vertical box
        vbox = gtk.VBox (False, 0)

        self.accelgroup = gtk.AccelGroup ()
        self.window.add_accel_group (self.accelgroup)

        menubar = self.createMenu (self.accelgroup)

        action = self.actiongroup.get_action ('Repeat')
        action.set_active (self.prefs.getRepeat())

        vbox.pack_start (menubar, False, True, 0)
        menubar.show ()

        # controls
        hbox = gtk.HBox (False, 0)
        vbox.pack_start (hbox, False, True, 0)
        hbox.show ()

        hbox2 = gtk.HBox (True, 0)
        hbox2.set_border_width (2)
        hbox.pack_start (hbox2, False, True, 0)
        hbox2.show ()
        
        # add buttons
        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_PREVIOUS, gtk.ICON_SIZE_BUTTON)
        self.prev_button = gtk.Button ("")
        self.prev_button.set_image (img)
        hbox2.pack_start (self.prev_button, True, True, 0)
        self.prev_button.connect ("clicked", self.prevCb)
        self.prev_button.show ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON)
        self.play_button = gtk.Button ("")
        self.play_button.set_image (img)
        hbox2.pack_start (self.play_button, True, True, 0)
        self.play_button.connect ("clicked", self.playCb)
        self.play_button.show ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON)
        self.pause_button = gtk.Button ("")
        self.pause_button.set_image (img)
        hbox2.pack_start (self.pause_button, True, True, 0)
        self.pause_button.connect ("clicked", self.pauseCb)
        self.pause_button.hide ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_BUTTON)
        self.stop_button = gtk.Button ("")
        self.stop_button.set_image (img)
        hbox2.pack_start (self.stop_button, True, True, 0)
        self.stop_button.connect ("clicked", self.stopCb)
        self.stop_button.show ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_NEXT, gtk.ICON_SIZE_BUTTON)
        self.next_button = gtk.Button ("")
        self.next_button.set_image (img)
        hbox2.pack_start (self.next_button, True, True, 0)
        self.next_button.connect ("clicked", self.nextCb)
        self.next_button.show ()

        self.setButtonsSensitivy (False, False, False, False)

        # sid controlls
        self.sidbox = gtk.HBox (False, 0)
        hbox.pack_start (self.sidbox, False, True, 0)
        self.sidbox.hide ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_REWIND, gtk.ICON_SIZE_BUTTON)
        button = gtk.Button ("")
        button.set_image (img)
        self.sidbox.pack_start (button, True, True, 0)
        button.connect ("clicked", self.sidPrevCb)
        button.show ()

        self.sidtrack_label = gtk.Label (" 1 ")
        self.sidbox.pack_start (self.sidtrack_label, True, True, 0)
        self.sidtrack_label.show ()

        img = gtk.image_new_from_stock (gtk.STOCK_MEDIA_FORWARD, gtk.ICON_SIZE_BUTTON)
        button = gtk.Button ("")
        button.set_image (img)
        self.sidbox.pack_start (button, True, True, 0)
        button.connect ("clicked", self.sidNextCb)
        button.show ()
        #end of sid controls

        # media length
        hbox2 = gtk.HBox (False, 0)
        hbox.pack_start (hbox2, True, True, 0)
        hbox2.show ()

        # TODO
        self.progress = gtk.ProgressBar ()
        self.progress.set_fraction (0)
        self.progress.set_text ("")
        self.progress.props.events = gtk.gdk.BUTTON_MOTION_MASK | gtk.gdk.BUTTON_PRESS_MASK | \
            gtk.gdk.BUTTON_RELEASE_MASK

        self.progress.connect ("button-press-event", self.progressButtonPressCb)
        self.progress.connect ("button-release-event", self.progressButtonReleaseCb)
        self.progress.connect('motion-notify-event', self.progressMotionNotifyCb)

        hbox2.pack_start (self.progress, True, True, 0)
        self.progress.show ()

        # volume
        button = gtk.VolumeButton ()
        button.set_value (volume)
        button.connect ("value-changed", self.volumeValueChanged)
        hbox.pack_start (button, False, True, 0)
        button.show ()
        

        # information area
        align = gtk.Alignment (0.0, 0.5, 0.0, 0.0)
        vbox.pack_start (align, False, False, 0)
        align.show ()

        self.title_label = gtk.Label ("<big><b>%s</b></big>" % "Title")
        self.title_label.set_use_markup (True)
        align.add (self.title_label)
        self.title_label.show ()

        hbox2 = gtk.HBox (False, 0)
        hbox2.set_border_width (2)
        vbox.pack_start (hbox2, False, True, 0)
        hbox2.show ()

        align = gtk.Alignment (0.0, 0.5, 0.0, 0.0)
        hbox2.pack_start (align, False, False, 0)
        align.show ()

        self.artist_label = gtk.Label ("Artist")
        align.add (self.artist_label)
        self.artist_label.show ()

        align = gtk.Alignment (0.0, 0.5, 0.0, 0.0)
        hbox2.pack_start (align, False, False, 0)
        align.show ()
        self.album_label = gtk.Label (" <i>(Album)</i>")
        self.album_label.set_use_markup (True)
        align.add (self.album_label)
        self.album_label.show ()

        # playlist area
        swindow = gtk.ScrolledWindow ()
        swindow.set_policy (gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        vbox.pack_start (swindow, True, True, 0)
        swindow.show ()

        self.medialist = MedialistTreeview (configdir, "playlist.db")
        # medialists callbacks to interesting signals
        self.medialist.connect ("row-activated", self.mediaActivatedCb)
        self.medialist.connect ("button-release-event", self.mediaSelectedCb)

        # popup menu
        self.medialist.connect ("button-press-event", self.buttonPressEventCb)
#        self.medialist.connect ("popup-menu", self.popupMenuHandlerCb)

        # added signals
        self.medialist.connect ("dnd-uri-add", self.dndUriAddCb)

        model = self.medialist.get_model ()
        model.connect ("row-deleted", self.rowDeleted, None)
        model.connect ("row-inserted", self.rowInserted, None)

        treeselection = self.medialist.get_selection ()
        treeselection.connect ("changed", self.selectionChangedCb, None)

        self.medialist.show ()
        swindow.add (self.medialist)

        # statusbar
        hbox2 = gtk.HBox (False, 0)
        hbox2.set_border_width (2)
       
        self.status_label = gtk.Label ("")
        hbox2.pack_start (self.status_label, False, True, 0)
        self.status_label.show ()

        vbox.pack_end (hbox2, False, True, 0)
        hbox2.show ()
 
        self.window.add (vbox);
        vbox.show ()

        action = self.actiongroup.get_action ('Suffle')
        action.set_active (self.prefs.getSuffle())

        self.window.connect ("delete_event", self.deleteEventCb)
        self.window.connect ("destroy", self.destroyCb)
        self.window.show ()
        try:
            pynotify.init ("KilikaliNG")
        except:
            pass

    def run (self):
        """
        run ()
        
        input: -
        output: -
 
        Actually only public method of the class. Inits some stuff,
        start workers and finally starts gtk.main ()
        """
        gtk.gdk.threads_init ()
        glib.idle_add (self.startWorkers)

        gtk.main ()


    def deleteEventIdle (self, widget):
        self.deleteEventCb (widget, None)
        return False

    def deleteEventCb (self, widget, data):
        # TODO: Is asking confirmation to quit needeed???
        self.quitCb (widget)
        return False

    def destroyCb (self, widget):
        None

    def startWorkers (self):
        self.import_worker = MediaImporter (self.db.basedir, self.db.dbname)
        self.import_worker.connect ("playlist-uri", self.importWorkerPlaylistUriCb)
        self.import_worker.connect ("add", self.importWorkerAddCb)
        self.import_worker.connect ("ready", self.importWorkerReadyCb)
        self.import_worker.start ()

        items = self.db.getItems ()

        glib.idle_add (self.medialist.addItemsToList, items)
        glib.idle_add (self.restoreCurrentPathIdle)
        glib.idle_add (self.addUriListIdle, self.cmdline.getUriList ())

        return False


    def addUriListIdle (self, uris):
        for uri in uris:
            self.addUri (uri, None)
        return False

    def restoreCurrentPathIdle (self):
        current_path = self.prefs.getCurrentPath ()
        self.medialist.setCurrentPath (current_path)
        return False

    def popupMenuDestroy (self, menu, data):
        menu.destroy ()

    def createPopupMenu (self, widget, event):
        actiongroup = self.actiongroup

        menu = gtk.Menu ()

        event_time = 0
        if event != None:
            button = event.button;
            event_time = event.time;
        else:
            button = None;
            event_time = gtk.get_current_event_time ();

        if self.pause_mode == False:
            action = actiongroup.get_action ('Play')
        else:
            action = actiongroup.get_action ('Pause')
        item = action.create_menu_item ()
        menu.append (item)

        action = actiongroup.get_action ('Stop')
        item = action.create_menu_item ()
        menu.append (item)

        action = actiongroup.get_action ('Next')
        item = action.create_menu_item ()
        menu.append (item)

        action = actiongroup.get_action ('Prev')
        item = action.create_menu_item ()
        menu.append (item)

        item = gtk.SeparatorMenuItem ()
        item.show ()
        menu.append (item)

        action = actiongroup.get_action ('Jump')
        item = action.create_menu_item ()
        menu.append (item)

        item = gtk.SeparatorMenuItem ()
        item.show ()
        menu.append (item)

        # edit->remove
        action = actiongroup.get_action ('Remove')
        item = action.create_menu_item ()
        menu.append (item)

        # edit->clear
        action = actiongroup.get_action ('Clear')
        item = action.create_menu_item ()
        menu.append (item)

        menu.attach_to_widget (widget, None)
        menu.popup (None, None, None, button, event_time)

    def volumeValueChanged (self, button, value):
        self.media.setVolume (value)

    def progressButtonPressCb (self, progress, event):
        current_item = self.medialist.current_item
        if current_item == None or current_item.duration_i == 0:
            return

        self.seeking = True

    def progressButtonReleaseCb (self, progress, event):
        self.seeking = False
        current_item = self.medialist.current_item
        if current_item == None or current_item.duration_i == 0:
            return

        self.getSeekFromProgress (progress, event)

        self.media.seekTo (self.seek_to)
        self.seek_to = 0

    def progressMotionNotifyCb (self, progress, event):
        current_item = self.medialist.current_item
        if current_item == None or current_item.duration_i == 0:
            return

        self.getSeekFromProgress (progress, event)

    def getSeekFromProgress (self, progress, event):
        mouse_x, mouse_y = event.get_coords ()
        allocation = self.progress.get_allocation ()

        frac = mouse_x / allocation.width
        if frac < 0:
            frac = 0
        if frac > 1:
            frac = 1
        self.progress.set_fraction (frac)

        current_item = self.medialist.current_item
        self.seek_to = current_item.duration_i * frac
        if self.seek_to == 0:
            self.seek_to = 1

    def buttonPressEventCb (self, widget, event):
        if event.button == 3 and event.type == gtk.gdk.BUTTON_PRESS:
            self.createPopupMenu (widget, event)
            return True
        return False

#    def popupMenuHandlerCb (self, widget):
#        print "popupMenuHandlerCb"
#        self.createPopupMenu (widget, event)
#        return True

    def createMenu (self, accelgroup):
        actiongroup = gtk.ActionGroup ('main')
        actiongroup.add_actions (self.basicitems)
        actiongroup.add_toggle_actions (self.toggleactions)

        for action in actiongroup.list_actions ():
            action.set_accel_group (accelgroup)

        menubar = gtk.MenuBar ()

        # file
        menuitem = actiongroup.get_action ('File').create_menu_item ()
        menubar.append (menuitem)
        menu = gtk.Menu ()
        menuitem.set_submenu (menu)

        # file->add files
        action = actiongroup.get_action ('Add')
        item = action.create_menu_item ()
        menu.append (item)

        # file->add directory
        action = actiongroup.get_action ('AddDir')
        item = action.create_menu_item ()
        menu.append (item)

        # file->add URI
        action = actiongroup.get_action ('AddURI')
        item = action.create_menu_item ()
        menu.append (item)

        item = gtk.SeparatorMenuItem ()
        item.show ()
        menu.append (item)

        # file->quit
        action = actiongroup.get_action ('Quit')
        item = action.create_menu_item ()
        menu.append (item)

        # edit
        menuitem = actiongroup.get_action ('Edit').create_menu_item ()
        menubar.append (menuitem)
        menu = gtk.Menu ()
        menuitem.set_submenu (menu)

        # edit->repeat
        action = actiongroup.get_action ('Repeat')
        item = action.create_menu_item ()
        menu.append (item)

        # edit->suffle
#        action = actiongroup.get_action ('Suffle')
#        item = action.create_menu_item ()
#        menu.append (item)

        item = gtk.SeparatorMenuItem ()
        item.show ()
        menu.append (item)

        # edit->remove
        action = actiongroup.get_action ('Remove')
        action.set_sensitive (False)
        item = action.create_menu_item ()
        menu.append (item)

        # edit->clear
        action = actiongroup.get_action ('Clear')
        action.set_sensitive (False)
        item = action.create_menu_item ()
        menu.append (item)

        # edit->preferences
#        action = actiongroup.get_action ('Prefs')
#        action.set_sensitive (False)
#        item = action.create_menu_item ()
#        menu.append (item)

        # playback
        menuitem = actiongroup.get_action ('Playback').create_menu_item ()
        menubar.append (menuitem)
        menu = gtk.Menu ()
        menuitem.set_submenu (menu)

        # playback->play
        action = actiongroup.get_action ('Play')
        item = action.create_menu_item ()
        menu.append (item)

        # playback->pause
        action = actiongroup.get_action ('Pause')
        action.set_visible (False)
        item = action.create_menu_item ()
        menu.append (item)

        # playback->stop
        action = actiongroup.get_action ('Stop')
        item = action.create_menu_item ()
        menu.append (item)

        # playback->next
        action = actiongroup.get_action ('Next')
        item = action.create_menu_item ()
        menu.append (item)

        # playback->prev
        action = actiongroup.get_action ('Prev')
        item = action.create_menu_item ()
        menu.append (item)

        item = gtk.SeparatorMenuItem ()
        item.show ()
        menu.append (item)

        # playback->jump
        action = actiongroup.get_action ('Jump')
        action.set_sensitive (False)
        item = action.create_menu_item ()
        menu.append (item)

        # about
        menuitem = actiongroup.get_action ('Help').create_menu_item ()
        menubar.append (menuitem)
        menu = gtk.Menu ()
        menuitem.set_submenu (menu)

        # about->help
        action = actiongroup.get_action ('About')
        item = action.create_menu_item ()
        menu.append (item)

        for action in actiongroup.list_actions ():
            action.set_property ('tooltip', action.get_property ('tooltip'))

        self.actiongroup = actiongroup
        menubar.show_all ()

        return menubar

    def setPauseMode (self, mode):
        pause_action = self.actiongroup.get_action ('Pause')
        play_action = self.actiongroup.get_action ('Play')

        if mode == True:
            pause_action.set_visible (True)
            play_action.set_visible (False)
            self.play_button.hide ()
            self.pause_button.show ()
            self.pause_mode = True
        else:
            pause_action.set_visible (False)
            play_action.set_visible (True)
            self.pause_button.hide ()
            self.play_button.show ()
            self.pause_mode = False

    def setButtonsSensitivy (self, prev, next, play, stop):
        self.prev_button.set_sensitive (prev)
        action = self.actiongroup.get_action ('Prev')
        action.set_sensitive (prev)

        self.next_button.set_sensitive (next)
        action = self.actiongroup.get_action ('Next')
        action.set_sensitive (next)

        self.play_button.set_sensitive (play)
        action = self.actiongroup.get_action ('Play')
        action.set_sensitive (play)

        self.stop_button.set_sensitive (stop)
        action = self.actiongroup.get_action ('Stop')
        action.set_sensitive (stop)

    # signal callbacls
    def setCurrentTimeCb (self, media, time, time_i, data):
        if self.seeking == True:
            return

        time_str = time
        current_item = self.medialist.current_item
        if current_item.duration != "":
            time_str += " / "
            time_str += current_item.duration
        self.progress.set_text (time_str)

        if (current_item.duration_i > 0):
            frac = float (time_i)/float (current_item.duration_i)
            self.progress.set_fraction (frac)

    # tag updating for net radios
    def tagUpdateCb (self, media, artist, title, data):
#        print 'tagUpdate'
#        print 'artist:', artist
#        print 'title:', title

        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref == None:
            return

        current_item = self.medialist.current_item
        if current_item.mediacontent.find ("audio/prs.sid") != 0 and \
                media.uri.find ('file://') == 0:
            return


        path = tree_row_ref.get_path ()
        if path == None:
            return
        model = tree_row_ref.get_model ()
        iter = model.get_iter (path)

        id = model.get_value (iter, 3)
        item = self.db.getItemById (id)
        if artist == "":
            artist = item.artist
        if title == "":
            title = item.title

        if artist == "" and title == "":
            return

        item.artist = artist
        item.title = title
        self.db.updateIdWithItem (id, item)

        self.medialist.setRowActive (model, iter, True)
        self.artist_label.set_label (artist)
        self.title_label.set_label ("<big><b>%s</b></big>" % glib.markup_escape_text (title))


    def eosCb (self, media, some, data):
        current_item = self.medialist.current_item
        if current_item.fullpath.find ('http://') == 0:
            self.playCb (None)
            return

        self.progress.set_fraction (0.0)
        self.progress.set_text ("")
        self.setPauseMode (False)
        self.previous_mode == ""

        if self.setNextSong () == False:
            return
        self.playCb (None)

    def errorCb (self, media, error, data):
        current_item = self.medialist.current_item
        if current_item.fullpath.find ('http://') == 0:
            self.playCb (None)
            return
        print "error:", error

        self.previous_mode = "error"

        # Skip to the next song
        glib.idle_add (self.nextCb, None)

    def dndUriAddCb (self, treeview, uri):
        self.addUri (uri, None)

    def addUri (self, uri, parent):
        # unquote uri (%20->' ' etc.)
        unquoted_uri = urllib.unquote (uri)
        ctype = None
        try:
            fp = gio.File (unquoted_uri)
        except:
            return False

        type = gio.FILE_TYPE_REGULAR
        try:
            info = fp.query_info ("*")
            type = info.get_file_type ()
            ctype = info.get_content_type ()

            # HACK - seems that sometimes playlists are octect-streams
            if ctype == "application/octet-stream":
                if unquoted_uri.find (".pls") > 0:
                    ctype = "audio/x-scpls"
                elif unquoted_uri.find (".m3u") > 0:
                    ctype = "audio/x-mpegurl"
                elif unquoted_uri.find (".asx") > 0:
                    ctype = "audio/x-ms-asx"
        except:
            pass

        # m3u and pls
        if ctype == "audio/x-mpegurl" or \
                ctype == "audio/x-scpls" or \
                ctype == "audio/x-ms-asx":
            self.import_worker.addPlaylist (unquoted_uri)
        elif type == gio.FILE_TYPE_DIRECTORY:
            self.import_worker.addDirectory (fp.get_path ())
        elif type == gio.FILE_TYPE_REGULAR:
            self.import_worker.addFile (unquoted_uri)

        return False

    def mediaSelectedCb (self, treeview, data):
        treeselection = treeview.get_selection ()
        if treeselection == None:
            return

        (model, paths) = treeselection.get_selected_rows ()
        if model == None or len (paths) == 0:
            return
        iter = model.get_iter (paths[0])

        treeselection.select_iter (iter)
        item = model.get_value (iter, 2)

    def rowInserted (self, model, path, iter, data):
        action = self.actiongroup.get_action ('Clear')
        action.set_sensitive (True)

    def rowDeleted (self, model, path, data):
        if len (model) == 0:
            self.setButtonsSensitivy (False, False, False, False)
            self.medialist.tree_row_ref == None
            action = self.actiongroup.get_action ('Clear')
            action.set_sensitive (False)

    def selectionChangedCb (self, treeselection, data):
        play = True
        remove = True

        tree_row_ref = self.medialist.getTreeRowRef ()

        (model, paths) = treeselection.get_selected_rows ()
        if len (paths) == 0 and tree_row_ref == None: 
            play = False
        if len (paths) == 0:
            remove = False

        self.setButtonsSensitivy (self.prev_button.get_property ("sensitive"),
                                  self.next_button.get_property ("sensitive"),
                                  play,
                                  self.stop_button.get_property ("sensitive"))

        remove_action = self.actiongroup.get_action ('Remove')
        remove_action.set_sensitive (remove)

    def mediaActivatedCb (self, treeview, editing, data):
#        self.stopCb (None)

        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref != None:
            path = tree_row_ref.get_path ()
            if path != None:
                model = tree_row_ref.get_model ()
                iter = model.get_iter (path)
                self.medialist.setRowActive (model, iter, False)

        treeselection = self.medialist.get_selection ()
        (model, paths) = treeselection.get_selected_rows ()
        if len (paths) == 0:
            return
        iter = model.get_iter (paths[0])
        self.medialist.setTreeRowRef (model, iter)

        self.playCb (None)

    def setPrevSong (self):
        iter = None
        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref != None:
            path = tree_row_ref.get_path ()
            model = tree_row_ref.get_model ()
            if self.prefs.getSuffle () == True:
                niter = self.medialist.getPrevIterFromSuffleList ()
                if niter != None:
                    self.medialist.setTreeRowRef (model, niter)
                if self.medialist.getSufflePrevListLen () == 0:
                    self.setButtonsSensitivy (False,
                                              self.next_button.get_property ("sensitive"),
                                              self.play_button.get_property ("sensitive"),
                                              self.stop_button.get_property ("sensitive"))

            niter = model.get_iter (path)
            self.medialist.setRowActive (model, niter, False)
            npath = path[0]-1
            if npath >= 0:
                iter = model.get_iter ( (npath,))
            else:
                if self.prefs.getRepeat () == True:
                    iter = model.get_iter ( (len (model)-1,))
                    if self.medialist.getSufflePrevListLen () == 0:
                        self.setButtonsSensitivy (False,
                                                  self.next_button.get_property ("sensitive"),
                                                  self.play_button.get_property ("sensitive"),
                                                  self.stop_button.get_property ("sensitive"))
                else:
                    self.setButtonsSensitivy (False,
                                              self.next_button.get_property ("sensitive"),
                                              self.play_button.get_property ("sensitive"),
                                              self.stop_button.get_property ("sensitive"))
        else:
            return False

        if iter == None:
            return False

        self.medialist.setTreeRowRef (model, iter)
        return True



    def setNextSong (self):
        iter = None
        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref != None:
            path = tree_row_ref.get_path ()
            model = tree_row_ref.get_model ()
            if path == None:
                path = (0, )
            iter = model.get_iter (path)
            self.medialist.setRowActive (model, iter, False)

            if self.prefs.getSuffle () == True:
                iter = self.medialist.getNextIterFromSuffleList (self.prefs.getRepeat ())
            else:
                iter = model.iter_next (iter)

            if iter == None:
                if self.prefs.getRepeat () == False:
                    return False
                iter = model.get_iter_first ()

        elif self.prefs.getSuffle () == True:
            print "setNext... getSuffle"
            model = self.medialist.get_model ()
            iter = self.medialist.getNextIterFromSuffleList (self.prefs.getRepeat ())
            if iter == None:
#                self.medialist.setRowActive (model, iter, True)
#            else:
                return False

        if iter == None:
            return False
        if self.medialist.getSufflePrevListLen () > 0:
            self.setButtonsSensitivy (True,
                                      self.next_button.get_property ("sensitive"),
                                      self.play_button.get_property ("sensitive"),
                                      self.stop_button.get_property ("sensitive"))

        self.medialist.setTreeRowRef (model, iter)

        return True

    def playCb (self, button):
        action = self.actiongroup.get_action ('Jump')
        action.set_sensitive (True)

        if self.previous_mode == "pause":
            self.previous_mode == "play"
            self.media.play ()
            self.setButtonsSensitivy (True, True, True, True)
            self.setPauseMode (True)
            return
        self.previous_mode == "play"


        if self.prefs.getSuffle () == True:
            if self.medialist.getSuffleListLen () == None:
                self.medialist.generateSuffleList ()
                self.setNextSong ()

        iter = None
        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref != None:
            path = tree_row_ref.get_path ()
            model = tree_row_ref.get_model ()
            iter = model.get_iter (path)
        else:
            treeselection = self.medialist.get_selection ()
            (model, paths) = treeselection.get_selected_rows ()
            if len (paths) == 0:
                return
            iter = model.get_iter (paths[0])
        if iter == None:
            return

        id = model.get_value (iter, 3)
        self.medialist.current_item = self.db.getItemById (id)
        current_item = self.medialist.current_item
        current_item.id = id

        if current_item.mediacontent == "audio/prs.sid":
            self.sidbox.show ()
            self.media.setSidMode (True)
            glib.idle_add (self.getSidTrack)
        else:
            self.sidbox.hide ()
            self.media.setSidMode (False)

        self.media.setUri (current_item.fullpath)
        self.media.play ()

        self.setSongInfoFromItem (current_item)
        self.medialist.setRowActive (model, iter, True)

        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref == None:
            self.medialist.setTreeRowRef (model, iter)

        self.setButtonsSensitivy (True, True, True, True)
        self.setPauseMode (True)

        title = "%s\n%s\n(%s)"%(current_item.title,
                                current_item.artist,
                                current_item.duration)

        if self.previous_mode == "prev":
            icon = "notification-audio-previous"
        elif  self.previous_mode == "next":
            icon = "notification-audio-next"
        else:
            icon = "notification-audio-play"
        self.showNotification ("Now playing...", title, icon)

#        print current_item.mediacontent

    def showNotification (self, title, body, type):
        try:
            if self.prefs.getNotifications () == True:
                if self.notification == None:
                    self.notification = pynotify.Notification (title, body, type)
                else:
                    self.notification.update (title, body, type)
                self.notification.show ()

        except:
            pass

    def getSidTrack (self):
        track = "%03d"%self.media.sidGetTrack ()
        self.sidtrack_label.set_text (track)
        return False

    def setSongInfoFromItem (self, item):
        current_item = self.medialist.current_item
        time_str = "00:00"
        if current_item.duration != "":
            time_str += " / "
            time_str += current_item.duration
        self.progress.set_text (time_str)

        if item.artist != "" and item.title != "":
            self.artist_label.set_label (item.artist)
            self.title_label.set_label ("<big><b>%s</b></big>" %
                                        glib.markup_escape_text (item.title))
        else:
            self.title_label.set_label ("<big><b>%s</b></big>" % 
                                        glib.markup_escape_text (item.filename))
            self.artist_label.set_label ("")

        if current_item.album != "":
            self.album_label.set_label (" <i>(%s)</i>" %
                                        glib.markup_escape_text (item.album))
        else:
            self.album_label.set_label ("" )

    def prevCb (self, button):
#        self.stopCb (None)
        try:
            self.notification.close ()
        except:
            pass

        self.media.stop () 
        self.setPrevSong ()
        self.previous_mode = "prev"
        self.playCb (None)

    def nextCb (self, button):
        self.media.stop () 
        try:
            self.notification.close ()
        except:
            pass
        if self.setNextSong () == False:
            self.stopCb (None)
            return False
        self.previous_mode = "next"
        self.playCb (None)
        return False

    def stopCb (self, button):
        action = self.actiongroup.get_action ('Jump')
        action.set_sensitive (False)

        self.media.stop ()
        self.progress.set_fraction (0.0)
        self.progress.set_text ("")
        self.setPauseMode (False)
        self.setButtonsSensitivy (False,
                                  False,
                                  self.play_button.get_property ("sensitive"),
                                  False)
        self.previous_mode = "stop"
        self.medialist.clearSuffleList ()

    def pauseCb (self, button):
        self.media.pause ()
        self.setPauseMode (False)
        self.previous_mode = "pause"

    def repeatCb (self, button):
        value = button.get_active ()
        self.prefs.setRepeat (value)

        # generate suffle list again if there is none,
        # so user is not so confused
        if self.prefs.getSuffle () == True:
            if self.medialist.getSuffleListLen () == 0:
                self.medialist.generateSuffleList ()

    def suffleCb (self, button):
        value = button.get_active ()
        self.prefs.setSuffle (value)
        if value == False:
            self.medialist.clearSuffleList ()

    def jumpCb (self, button):
        tree_row_ref = self.medialist.getTreeRowRef ()
        if tree_row_ref == None:
            return

        path = tree_row_ref.get_path ()
        self.medialist.scroll_to_cell (path)

    def sidNextCb (self, button):
        self.media.sidNextTrack ()
        glib.idle_add (self.getSidTrack)

    def sidPrevCb (self, button):
        self.media.sidPrevTrack ()
        glib.idle_add (self.getSidTrack)

    def createPlaylistFromTreeview (self, treeview):
        model = treeview.get_model ()
        iter = model.get_iter_first ()
        pl = Playlist ()
        while iter:
            id = model.get_value (iter, 3)
            item = self.db.getItemById (id)
            if item.fullpath != '':
                pl.addFile (item.fullpath)
            iter = model.iter_next (iter)
        return pl 

    def dbusControlPlayCb (self, obj, b):
        print "dbusControlPlayCb"
        if self.play_button.get_property ("sensitive") == False and \
                self.pause_mode == False:
            return
        self.playCb (None)

    def dbusControlStopCb (self, obj, b):
        if self.stop_button.get_property ("sensitive") == False:
            return
        self.stopCb (None)

    def dbusControlPauseCb (self, obj, b):
        if self.pause_button.get_property ("sensitive") == False and \
                self.pause_mode == True:
            return
        self.pauseCb (None)

    def dbusControlPreviousCb (self, obj, b):
        if self.prev_button.get_property ("sensitive") == False:
            return
        self.prevCb (None)

    def dbusControlNextCb (self, obj, b):
        if self.next_button.get_property ("sensitive") == False:
            return
        self.nextCb (None)

    def dbusAddUriListCb (self, obj, uri_list):
        glib.idle_add (self.addUriListIdle, uri_list)

    def dbusQuitCb (self, obj, name):
        print "dbusQuitCb"
        glib.idle_add (deleteEventIdle, None)


    # actiongroup callbacks
    # quit
    def quitCb (self, widget):
        try:
            self.notification.close ()
        except:
            pass

        # save state
        width, height = self.window.get_size ()
        x, y = self.window.get_position ()
        self.prefs.setWindowAllocation (x, y, width, height)

        volume = self.media.getVolume ()
        self.prefs.setVolume (volume)

        current_path = self.medialist.getCurrentPath ()
        self.prefs.setCurrentPath (current_path)

        if self.last_dir != None:
            self.prefs.setDirectoryPath (self.last_dir)

        self.prefs.saveState ()

        # join workers
        try:
            self.import_worker.forceQuit ()
            self.import_worker.join ()
        except:
            pass

        gtk.main_quit ()

    # add URI
    def addURICb (self, widget):
        uri = None
        dialog = gtk.Dialog ("Add URI...",
                             self.window,
                             gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                             (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                              gtk.STOCK_OK, gtk.RESPONSE_OK))
        vbox = dialog.get_content_area ()
        entry = gtk.Entry ()
        entry.show ()
        vbox.pack_start (entry, False, True, 0)
 
        response = dialog.run ()
        if response == gtk.RESPONSE_OK:
            uri = entry.get_text ()
        dialog.destroy ()
        if len (uri) > 0:
            glib.idle_add (self.addUri, uri, None)

    # add files to playlist
    def addFilesCb (self, widget):
        chooser = gtk.FileChooserDialog('Choose files',
                                        self.window,
                                        gtk.FILE_CHOOSER_ACTION_OPEN,
                                        (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        chooser.set_select_multiple (True)
        if self.last_dir != None:
            chooser.set_current_folder (self.last_dir)

        files = []
        response = chooser.run ()
        if response == gtk.RESPONSE_OK:
            directory = chooser.get_current_folder ()
            self.last_dir = directory

            files = chooser.get_uris ()
        chooser.destroy ()

        for fullpath in files:
            glib.idle_add (self.addUri, fullpath, None)
        del files


    # add all supported files from directory to playlist
    def addDirectoryCb (self, widget):
        directory = ''
        chooser = gtk.FileChooserDialog('Choose directory',
                                        self.window,
                                        gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                        (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        if self.last_dir != None:
            chooser.set_current_folder (self.last_dir)

        directory = None
        response = chooser.run ()
        if response == gtk.RESPONSE_OK:
            directory = chooser.get_uri ()
            self.last_dir = chooser.get_filename ()
        chooser.destroy()

        if directory != None:
            glib.idle_add (self.addUri, directory, None)


    def importWorkerPlaylistUriCb (self, worker, uri):
        self.addUri (uri, None)

    def importWorkerAddCb (self, worker, sitem):
        self.status_label.set_label ("Adding URI:s...")
        glib.idle_add (self.medialist.addItemToList, sitem)

    def importWorkerReadyCb (self, worker, sitem):
        self.status_label.set_label ("")

    def printItem (self, item):
        print "======================================"
        print "fullpath:", item.fullpath
        print "filename:", item.filename
        print "artist:", item.artist
        print "mediacontect:", item.mediacontent
        print "album:", item.album
        print "title:", item.title
        print "track_number:", item.track_number
        print "track_count", item.track_count
        print "duration:", item.duration
        print "genre:", item.genre
        print "audiocodec:", item.audio_codec
        print "bitrate:", item.bitrate
        print "videocodec", item.video_codec
        print "codec:", item.codec
        print "comment:", item.comment
        print "extended comment:", item.extended_comment
        print "track_gain:", item.track_gain
        print "track_peak:", item.track_peak
        print "album_gain", item.album_gain
        print "album_peak:", item.album_peak
        print "album_volume_count:", item.album_volume_count
        print "album_volume_number:", item.album_volume_number
        print "contact:", item.contact
        print "copyright:", item.copyright
        print "date:", item.date
        print "desc:", item.description
        print "serial:", item.serial
        print "encoder:", item.encoder
        print "encoder_version:", item.encoder_version
        print "max_bitrate:", item.maximum_bitrate
        print "min_bitrate:", item.minimum_bitrate
        print "nominal_bitrate:", item.nominal_bitrate
        print "preview_image:", item.previev_image
        print "image:", item.image
        print "isrc:", item.isrc
        print "language_code:", item.language_code
        print "license:", item.license
        print "location:", item.location
        print "version:", item.version
        print "performer:", item.performer
        print "organization:", item.organization
        print "======================================"

    def removeCb (self, button):
        ids = []
        treeselection = self.medialist.get_selection ()
        (model, paths) = treeselection.get_selected_rows ()
        while len (paths) > 0:
            iter = model.get_iter (paths[0])
            id = model.get_value (iter, 3)
            item = self.db.getItemById (id)

            current_item = self.medialist.current_item
            if current_item != None and id == current_item.id:
                self.stopCb (None)
                self.medialist.clearTreeRowRef ()

            ids.append (id)
            model.remove (iter)
            (model, paths) = treeselection.get_selected_rows ()
        self.db.removeByIdList (ids)

    def clearCb (self, button):
        self.stopCb (None)
        self.medialist.clearTreeRowRef ()
        model = self.medialist.get_model ()
        model.clear ()
        self.db.removeAll ()

    def prefsCb (self, button):
        # TODO
        prefs.run ()
        prefs.destroy ()

    # about
    def aboutCb (self, widget):
        about = gtk.AboutDialog ()
        about.set_name ("KilikaliNG")
        about.set_version (VERSION)
        about.set_copyright ("2009 Joni Valtanen")
        about.set_license ("GNU General Public License version 2")
        about.set_website ("http://koti.kapsi.fi/~jvaltane/kilikaling/")
        about.set_authors (("Joni Valtanen",))
        about.run ()
        about.destroy ()
