import dbus
import gobject
import dbus.service
from dbus.mainloop.glib import DBusGMainLoop
from _dbus_bindings import *

DBUS_NAME = "org.kilikaling"
DBUS_IFACE = "org.kilikaling"
DBUS_PATH = "/org/kilikaling"

main_loop = DBusGMainLoop ()
bus = dbus.SessionBus (mainloop=main_loop)

class DBusObj (gobject.GObject):
    __gsignals__ = { 'quit': ( gobject.SIGNAL_RUN_FIRST,
                               gobject.TYPE_NONE,
                               (gobject.TYPE_STRING,)),
                     'control-play': ( gobject.SIGNAL_RUN_FIRST,
                                       gobject.TYPE_NONE,
                                       (gobject.TYPE_BOOLEAN,)),
                     'control-stop': ( gobject.SIGNAL_RUN_FIRST,
                                       gobject.TYPE_NONE,
                                       (gobject.TYPE_BOOLEAN,)),
                     'control-pause': ( gobject.SIGNAL_RUN_FIRST,
                                        gobject.TYPE_NONE,
                                        (gobject.TYPE_BOOLEAN,)),
                     'control-previous': ( gobject.SIGNAL_RUN_FIRST,
                                           gobject.TYPE_NONE,
                                           (gobject.TYPE_BOOLEAN,)),
                     'control-next': ( gobject.SIGNAL_RUN_FIRST,
                                       gobject.TYPE_NONE,
                                       (gobject.TYPE_BOOLEAN,)),
                     'add-uri-list': ( gobject.SIGNAL_RUN_FIRST,
                                       gobject.TYPE_NONE,
                                       (gobject.TYPE_PYOBJECT,))}

class KilikaliNGDBus (dbus.service.Object):
    def __init__ (self):
        self.obj = DBusObj ()
        self.shutting_down = False
        global bus
        dbus.service.Object.__init__ (self, bus, DBUS_PATH)

        bus_obj = bus.get_object (dbus.BUS_DAEMON_NAME, dbus.BUS_DAEMON_PATH)
        bus_obj.connect_to_signal ("NameLost", self.NameLostCb, dbus.BUS_DAEMON_IFACE)

    # Signal handlers
    def NameLostCb (self, bus_name):
        """
        "NameLost" signal handler. This happens when Primary ownership of the bus name
        is losted.
        
        emits "quit" signal so application can quit.
        """
        #FIXME: check is checking bus name mandatory?
        #FIXME: find better way to emit signals
        if bus_name == DBUS_NAME:
            self.obj.emit ("quit", bus_name)

    # method calls
    @dbus.service.method(dbus_interface=DBUS_IFACE, in_signature='as')
    def addUriList (self, uri_list):
        self.obj.emit ("add-uri-list", uri_list)

    @dbus.service.method(dbus_interface=DBUS_IFACE)
    def play (self):
        self.obj.emit ("control-play", True)
    @dbus.service.method(dbus_interface=DBUS_IFACE)
    def stop (self):
        self.obj.emit ("control-stop", True)
    @dbus.service.method(dbus_interface=DBUS_IFACE)
    def pause (self):
        self.obj.emit ("control-pause", True)
    @dbus.service.method(dbus_interface=DBUS_IFACE)
    def next (self):
        self.obj.emit ("control-next", True)
    @dbus.service.method(dbus_interface=DBUS_IFACE)
    def previous (self):
        self.obj.emit ("control-previous", True)

def dbusIsPrimaryOwner (replace):
    """
    dbusIsPrimaryOwner ()

    input: @replace - boolean. If true replaces existing instance. To use with
                               "--replace" command line option
    output: boolean - got primary owner status

    isPrimaryOwner() tests if there is already running instance of application. Replacing
    existing is possible.
    """

    flags = NAME_FLAG_ALLOW_REPLACEMENT|NAME_FLAG_DO_NOT_QUEUE
    if replace == True:
        flags = flags|NAME_FLAG_REPLACE_EXISTING

    global bus
    bus = dbus.SessionBus ()
    retval = bus.request_name (DBUS_NAME, flags)

    if retval == REQUEST_NAME_REPLY_PRIMARY_OWNER:
        return True

    return False


def _getProxy ():
    bus = dbus.SessionBus ()
    proxy = bus.get_object (DBUS_NAME, DBUS_PATH)
    return proxy

def dbusAddUriList (uri_list):
    if len (uri_list) == 0:
        return
    try:
        _getProxy ().addUriList (uri_list)
    except:
        return

def dbusPlay ():
    try:
        _getProxy ().play ()
    except:
        return

def dbusStop ():
    try:
        _getProxy ().stop ()
    except:
        return

def dbusPause ():
    try:
        _getProxy ().pause ()
    except:
        return

def dbusNext ():
    try:
        _getProxy ().next ()
    except:
        return

def dbusPrevious ():
    try:
        _getProxy ().previous ()
    except:
        return
